shader = {

    canvas = {
        love.graphics.newCanvas(),
        love.graphics.newCanvas(),
    },

    effect = {

        ripples = love.graphics.newPixelEffect [[
            extern number t      = 0.0;
            extern number fac    = 0.0;
            uniform sampler2D tex0;
            vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
            {
                vec2 uv         = vec2(
                    texture_coords.x + cos(t + t * fac + gl_FragCoord.x/4)/(640-320*fac),
                    texture_coords.y + cos((t + t * fac)/1.69 + (1 - texture_coords.y*480)/4)/(480-240*fac)
                );
                vec3 col        = vec3(
                    texture2D(tex0,uv).x + .15,//*(1-fac/1.5) + fac/2 + .15,
                    texture2D(tex0,uv).y + .3,//*(1-fac/1.5) + fac/2 + .3,
                    texture2D(tex0,uv).z + .5//*(1-fac/1.5) + fac/2 + .4
                );
                //texture2D(tex0,uv).xyz*50.0/cLength;
             return vec4(col, 1.0);
            }
        ]],

        lightrays = love.graphics.newPixelEffect [[
            extern number exposure = 0.1;
            extern number decay = 0.9;
            extern number density = 0.1;
            extern number weight = 0.9;
            extern vec2 lightPositionOnScreen= vec2(.5,0.0);
            extern number NUM_SAMPLES = 16.0 ;
            vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
            {
                vec2 deltaTextCoord = vec2( texture_coords - lightPositionOnScreen.xy );
                vec2 textCoo = texture_coords.xy;
                deltaTextCoord *= 1.0 / float(NUM_SAMPLES) * density;
                float illuminationDecay = 1.0;
                vec4 cc = vec4(0.0, 0.0, 0.0, 1.0);
                for(int i=0; i < NUM_SAMPLES ; i++)
                {
                    textCoo -= deltaTextCoord;
                    vec4 sample = Texel( texture, textCoo );
                    sample *= illuminationDecay * weight;
                    cc += sample;
                    illuminationDecay *= decay;
                }
                cc *= exposure;
                return cc;
            }
        ]],

        sand = love.graphics.newPixelEffect [[
            extern number t      = 0.0;
            uniform sampler2D tex0;
            vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
            {
                vec2 uv         = vec2(
                    mod((texture_coords.x - .5)/atan(texture_coords.y/2) + mod(t/4,1),1),
                    1-texture_coords.y
                );
                vec3 col        = vec3(
                    texture2D(tex0,uv).x,
                    texture2D(tex0,uv).y,
                    texture2D(tex0,uv).z
                );
                //texture2D(tex0,uv).xyz*50.0/cLength;
             return vec4( col, 1.0 );
            }
        ]],

        sky = love.graphics.newPixelEffect [[
            extern number t      = 0.0;
            uniform sampler2D tex0;
            vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
            {
                vec2 uv         = vec2(
                    mod((texture_coords.x/2 - .25)/sin(1-texture_coords.y/2) + mod(t/512,1),1),
                    texture_coords.y*2 - 1
                );
                vec3 col        = vec3(
                    texture2D(tex0,uv).x,
                    texture2D(tex0,uv).y,
                    texture2D(tex0,uv).z
                );
                //texture2D(tex0,uv).xyz*50.0/cLength;
             return vec4( col, 1.0 );
            }
        ]],

        reflection = love.graphics.newPixelEffect [[
            extern number t      = 0.0;
            //extern number h      = 0.0;
            uniform sampler2D tex0;
            vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
            {
                vec2 uv2         = vec2(
                    texture_coords.x,
                    texture_coords.y
                );
                vec3 col2        = vec3(
                    texture2D(tex0,uv2).x,
                    texture2D(tex0,uv2).y,
                    texture2D(tex0,uv2).z
                );
                vec2 uv         = vec2(
                    texture_coords.x,
                    1 - texture_coords.y + fract(sin(dot(vec2(texture_coords.x, 1 - texture_coords.y)/8+ceil(t*16)/64 ,vec2(12.9898,78.233))) * 43758.5453)/128
                );
                vec3 col        = vec3(
                    texture2D(tex0,uv).x,
                    texture2D(tex0,uv).y,
                    texture2D(tex0,uv).z
                );
                //texture2D(tex0,uv).xyz*50.0/cLength;
             return vec4( col, .5);
            }
        ]],

        rain = love.graphics.newPixelEffect [[
            extern number t      = 0.0;
            extern number fac    = 0.0;
            uniform sampler2D tex0;
            vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
            {
                vec2 uv2         = vec2(
                    texture_coords.x,
                    texture_coords.y
                );
                vec3 col2        = vec3(
                    texture2D(tex0,uv2).x,
                    texture2D(tex0,uv2).y,
                    texture2D(tex0,uv2).z
                );
                vec2 uv         = vec2(
                    texture_coords.x,
                    texture_coords.y + (fract(sin(dot(vec2(texture_coords.x*10,texture_coords.y*7.5)+ceil(t*64)/64 ,vec2(12.9898,78.233))) * 43758.5453)/64) * fac
                );
                vec3 col        = vec3(
                    texture2D(tex0,uv).x + (fract(sin(dot(vec2(texture_coords.x*10,texture_coords.y*7.5)+ceil(t*64)/16 ,vec2(12.9898,78.233))) * 43758.5453)/4) * fac,
                    texture2D(tex0,uv).y + (fract(sin(dot(vec2(texture_coords.x*10,texture_coords.y*7.5)+ceil(t*64)/16 ,vec2(12.9898,78.233))) * 43758.5453)/4) * fac,
                    texture2D(tex0,uv).z + (fract(sin(dot(vec2(texture_coords.x*10,texture_coords.y*7.5)+ceil(t*64)/16 ,vec2(12.9898,78.233))) * 43758.5453)/4) * fac
                );
                //texture2D(tex0,uv).xyz*50.0/cLength;
             return vec4( col, 1 );
            }
        ]],

        mask = love.graphics.newPixelEffect [[
            extern Image mask;
            uniform sampler2D tex0;
            vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
            {
                vec4 mask_color = Texel( mask, texture_coords );
                vec4 col = vec4(
                    texture2D(tex0,texture_coords).x * color.r,
                    texture2D(tex0,texture_coords).y * color.g,
                    texture2D(tex0,texture_coords).z * color.b,
                    color.a - mask_color.r
                );
                //texture2D(tex0,texture_coords).xyz*50.0/cLength;
             return vec4(col);
            }
        ]],

        norm = love.graphics.newPixelEffect [[
            extern Image map;
            extern number fac = .1;
            uniform sampler2D tex0;
            vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
            {
                vec2 uv2         = vec2(
                        texture_coords.x,
                        1 - texture_coords.y
                );
                vec4 mapcol = Texel(map, uv2);
                vec2 uv         = vec2(
                        texture_coords.x + (.5 - mapcol.r)/(8/fac)*ceil(mapcol.r + mapcol.g + mapcol.b),
                        texture_coords.y + (.5 - mapcol.g)/(8/fac)*ceil(mapcol.r + mapcol.g + mapcol.b)
                );

                vec4 col = vec4(
                        texture2D(tex0,uv).x,
                        texture2D(tex0,uv).y,
                        texture2D(tex0,uv).z,
                        1
                );
             return vec4( col);
            }
        ]],

    },

    update = {
        ripples = function()
            shader.effect.ripples:send('t', game.timer * 4)
            shader.effect.ripples:send('fac', game.rain * (512/screen.w))
        end,
        sky = function()
            shader.effect.sky:send('t', game.timer*4)
        end,
        sand = function()
            shader.effect.sand:send('t', (512/screen.w)*game.timer)
        end,
        lightrays = function()
            shader.effect.lightrays:send('lightPositionOnScreen', {tero.x/screen.w,(screen.h-240)/screen.h})
        end,
        reflection = function()
            shader.effect.reflection:send('t', game.timer)
            --shader.effect.reflection:send('h', (240-screen.h)/screen.h)
        end,
        rain = function()
            shader.effect.rain:send('t', game.timer)
            shader.effect.rain:send('fac', (512/screen.w)*game.rain)
            shader.effect.lightrays:send('exposure', game.rain*.1)
        end,
    },

    draw = {

        ripples = function()
            love.graphics.setCanvas(shader.canvas[2])
            love.graphics.draw(shader.canvas[1], 0, 0)

            --ripples
            love.graphics.setScissor(0, screen.h-240, screen.w, 240)
            love.graphics.setPixelEffect(shader.effect.ripples)
            love.graphics.draw(shader.canvas[1], 0, 0)
            love.graphics.setPixelEffect()
            --reflection
            love.graphics.setScissor(0, screen.h-240, screen.w, 32)
            love.graphics.setPixelEffect(shader.effect.reflection)
            love.graphics.draw(shader.canvas[1], 0, screen.h-480, 0, 1, 1)
            love.graphics.setPixelEffect()
            love.graphics.setScissor()
            love.graphics.setCanvas(shader.canvas[1])
            love.graphics.draw(shader.canvas[2])
        end,

        sand = function()
            love.graphics.setPixelEffect(shader.effect.sand)
            love.graphics.draw(tex.sand, 0, screen.h-128, 0, screen.w/128, 1)
            love.graphics.setPixelEffect()
        end,

        sky = function()
            love.graphics.setPixelEffect(shader.effect.sky)
            love.graphics.draw(sky.day, 0, 0, 0, (screen.h-240)/240)
            love.graphics.setPixelEffect()
        end,

        reflection = function()
            --love.graphics.setBlendMode('alpha')
            --love.graphics.setPixelEffect(shader.effect.reflection)
            --love.graphics.circle('fill', love.mouse.getX(),love.mouse.getY(), 8, 8)
            --love.graphics.draw(shader.canvas[2], love.mouse.getX(), love.mouse.getY(), 0, .125, .125)
            --love.graphics.setPixelEffect()
            --love.graphics.setBlendMode('premultiplied')
        end,

        rain = function()
            if game.rain > 0 then
                love.graphics.draw(shader.canvas[2], 0, 0)
                love.graphics.setScissor(0, 0, screen.w, screen.h-240)
                love.graphics.setPixelEffect(shader.effect.rain)
                love.graphics.draw(shader.canvas[2], 0, 0)
                love.graphics.setPixelEffect()

                love.graphics.setScissor(0, screen.h-208, screen.w, 208)
                love.graphics.setPixelEffect(shader.effect.lightrays)
                love.graphics.draw(shader.canvas[2])
                love.graphics.setPixelEffect()
                love.graphics.setScissor()
            else
                love.graphics.draw(shader.canvas[2], 0, 0)
            end
        end,

        day = function()

            if game.flashlight and game.flashlighttimer > 0 then
                shader.canvas[2]:clear()
                love.graphics.setCanvas(shader.canvas[2])
                --Draws black & white image used to control strength of color changing thing
                --Y reversed because the shader flips the Y axis
                love.graphics.setColor(0, 0, 0)
                love.graphics.rectangle('fill', 0, 0, screen.w, screen.h)
                love.graphics.setColor(255, 255, 255)
                love.graphics.polygon('fill', tero.x + 64, 240, tero.x + 192, 0, tero.x - 64, 0)
                love.graphics.setCanvas(shader.canvas[1])
                shader.effect.mask:send('mask', shader.canvas[2])
                love.graphics.setPixelEffect(shader.effect.mask)
            else
                game.flashlight = false
            end

            --Time change
            love.graphics.setBlendMode('multiplicative')
            local tmp = math.mod(game.timer/32,32)
            local tmp2 = math.mod(tmp,1)
            local tmp3 = math.mod(tmp,1) * 255
            local tmp4 = 255 - tmp3
            local tmp5 = math.ceil(tmp)
            if tmp5 >= 1 and tmp5 < 8 then
                if tmp5 == 1 then
                    love.graphics.setColor(tmp4, tmp4, tmp4, tmp4)
                    love.graphics.draw(tex.overlay.sun, 0, 0, 0, screen.w/640, screen.h/480)
                    love.graphics.setColor(tmp3, tmp3, tmp3, tmp3)
                    love.graphics.draw(tex.overlay.day, 0, 0, 0, screen.w/640, screen.h/480)
                else
                    love.graphics.setColor(255, 255, 255, 255)
                    love.graphics.draw(tex.overlay.day, 0, 0, 0, screen.w/640, screen.h/480)
                end
            elseif tmp5 >= 8 and tmp5 < 16 then
                if tmp5 == 8 then
                    love.graphics.setColor(tmp4, tmp4, tmp4, tmp4)
                    love.graphics.draw(tex.overlay.day, 0, 0, 0, screen.w/640, screen.h/480)
                    love.graphics.setColor(tmp3, tmp3, tmp3, tmp3)
                    love.graphics.draw(tex.overlay.sun, 0, 0, 0, screen.w/640, screen.h/480)
                else
                    love.graphics.setColor(255, 255, 255, 255)
                    love.graphics.draw(tex.overlay.sun, 0, 0, 0, screen.w/640, screen.h/480)
                end
            elseif tmp5 >= 16 and tmp5 < 24 then
                if tmp5 == 16 then
                    love.graphics.setColor(tmp4, tmp4, tmp4, tmp4)
                    love.graphics.draw(tex.overlay.sun, 0, 0, 0, screen.w/640, screen.h/480)
                    love.graphics.setColor(tmp3, tmp3, tmp3, tmp3)
                    love.graphics.draw(tex.overlay.night, 0, 0, 0, screen.w/640, screen.h/480)
                else
                    love.graphics.setColor(255, 255, 255, 255)
                    love.graphics.draw(tex.overlay.night, 0, 0, 0, screen.w/640, screen.h/480)
                end
            elseif tmp5 >= 24 and tmp5 < 33 then
                if tmp5 == 24 then
                    love.graphics.setColor(tmp4, tmp4, tmp4, tmp4)
                    love.graphics.draw(tex.overlay.night, 0, 0, 0, screen.w/640, screen.h/480)
                    love.graphics.setColor(tmp3, tmp3, tmp3, tmp3)
                    love.graphics.draw(tex.overlay.sun, 0, 0, 0, screen.w/640, screen.h/480)
                else
                    love.graphics.setColor(255, 255, 255, 255)
                    love.graphics.draw(tex.overlay.sun, 0, 0, 0, screen.w/640, screen.h/480)
                end
            else
            end
            love.graphics.setColor(255,255,255,255)
            love.graphics.setBlendMode('alpha')
            love.graphics.setPixelEffect()
        end,

        hud = function()
            love.graphics.setColor(255 - game.score, game.score, game.score)
            love.graphics.arc('fill', 8, 20, 87, math.pi/2, math.pi/2 - game.score * (math.pi/512), 64)
            love.graphics.setColor(255,255,255)

            love.graphics.setColor(255 - game.flashlighttimer, game.flashlighttimer, game.flashlighttimer)
            love.graphics.arc('fill', screen.w, 20, 87, game.flashlighttimer * (math.pi/512) + math.pi/2, math.pi/2, 64)
            love.graphics.setColor(255,255,255)

            shader.canvas[2]:clear()
            love.graphics.setCanvas(shader.canvas[2])
            love.graphics.draw(tex.hudnorm, 0, screen.h, 0, 1, -1)
            love.graphics.draw(tex.hudnorm, screen.w, screen.h, 0, -1, -1)
            love.graphics.setCanvas(shader.canvas[1])
            shader.effect.norm:send('map', shader.canvas[2])
            love.graphics.setPixelEffect(shader.effect.norm)
            love.graphics.draw(shader.canvas[1], 0, 0)
            love.graphics.setPixelEffect()
            love.graphics.setBlendMode('premultiplied')
            love.graphics.draw(tex.hud, 0, 0)
            love.graphics.draw(tex.hud, screen.w, 0, 0, -1, 1)
            love.graphics.setBlendMode('alpha')
        end,
    },

}