require 'lib/maths'
require 'lib/shader'
require 'lib/alpha'
function love.load()

    settings = {
        music = false,
        sfx = true,
        fullscreen = false,
        modes = love.graphics.getModes(),
        --key config
        button = {
            up = {'w','up',','},
            down = {'s','down','o'},
            left = {'a','left'},
            right = {'d','right','e'},
            pull = {' '},
            flashlight = {'f', 'p'}
        },
    }

    table.sort(settings.modes, function(a, b) return a.width > b.width end)
    if settings.fullscreen then
        love.graphics.setMode(settings.modes[1].width,settings.modes[1].height, true, false)
    end

    if settings.music then
        music = {
            underwater = love.audio.newSource('sound/music/underwater.ogg')
        }
        music.underwater:setLooping(true)
        love.audio.play(music.underwater)
    end

    if settings.sfx then
        sfx = {
            --ambient rain
            rain = love.audio.newSource('sound/fx/rain.ogg'),
            --rain hitting the boat, plays in stereo depending on its position
            boatrain = love.audio.newSource('sound/fx/boatrain.ogg'),
            --when the hook hits the water
            splash = love.audio.newSource('sound/fx/splash.ogg', 'static'),
            --when the hook comes up out of the water
            up = love.audio.newSource('sound/fx/up.ogg', 'static'),
            --when a fish is snagged
            cut = love.audio.newSource('sound/fx/cut.ogg', 'static'),
            --when a fish hits the boat
            slap = love.audio.newSource('sound/fx/slap.ogg', 'static'),
            --reeling in
            reel = love.audio.newSource('sound/fx/reel.ogg', 'static'),
            --when the hook is in the air
            wind = love.audio.newSource('sound/fx/wind.ogg', 'static'),
            --when your boat moves
            surf = love.audio.newSource('sound/fx/surf.ogg', 'static'),
            --flashlight
            switch = love.audio.newSource('sound/fx/switch.ogg', 'static'),
            --the numbers are how many seconds its been playing
            --I need to reset it because calling love.audio.play()
            --too much freezes the game
            playing = {
                rain = false,
                --boatrain is tied to rain and thus doesnt need its own variable
                splash = 0,
                up = 0,
                cut = 0,
                slap = 0,
                reel = false,
                wind = false,
                surf = false,
                switch = 0,
            }
        }
        sfx.rain:setLooping(true)
        sfx.boatrain:setLooping(true)
        sfx.reel:setLooping(true)
        sfx.wind:setLooping(true)
        sfx.surf:setLooping(true)
    end

    debugging = true
    love.mouse.setVisible(false)

    --sets a red BG to make alpha issues more apparent
    love.graphics.setBackgroundColor(255, 0, 0, 255)
    love.graphics.setBlendMode('premultiplied')
    love.graphics.setLine(2, 'rough')

    --random game info
    game = {
        --total time passed
        timer = 0,
        --when new fish can spawn
        fishtimer = 0,
        --changes many aspects such as spawn rates and speed
        --difficulty = 0,
        --basically how much time you have left, you add to it when you catch fish.
        score = 255,
        totalscore = 0,
        --lights up water at the cost of visibility
        flashlight = false,
        --rain effect strength between 0 and 1
        rain = 0,
        --how much flashlight time you have left
        flashlighttimer = 255,
    }

    --more things are added in the game
    screen = {
        w = love.graphics.getWidth(),
        h = love.graphics.getHeight(),
    }

    --static textures
    tex = {
        sand = love.image.newImageData('tex/sand.png'),
        shadow = love.image.newImageData('tex/shadow.png'),
        float = love.image.newImageData('tex/float.png'),
        boat = love.image.newImageData('tex/boat.png'),
        splash = love.image.newImageData('tex/splash.png'),
        rock = love.image.newImageData('tex/rock.png'),
        hook = love.image.newImageData('tex/hook.png'),
        drag = love.image.newImageData('tex/drag.png'),
        hud = love.image.newImageData('tex/hud.png'),
    }
    premultiply(tex)
    tex.hudnorm = love.graphics.newImage('tex/hudnorm.png')

    --these are blended with multiply and dont need premultiplied
    tex.overlay = {
        sun = love.graphics.newImage('tex/overlay/sun.png'),
        day = love.graphics.newImage('tex/overlay/day.png'),
        night = love.graphics.newImage('tex/overlay/night.png'),
    }

    island = {
        love.image.newImageData('tex/island/1.png')
    }
    premultiply(island)

    sky = {
        --sun = love.image.newImageData('tex/sky/sun.png'),
        day = love.image.newImageData('tex/sky/day.png'),
        --night = love.image.newImageData('tex/sky/night.png'),
    }
    premultiply(sky)

    --animated textures
    anim = {
        fish = {
            tex = love.image.newImageData('tex/fish.png'),
        }
    }
    premultiply(anim.fish)


    --generates quads for multi frame images
    for k,v in pairs(anim) do
        local w = anim[k].tex:getWidth()
        local h = anim[k].tex:getHeight()
        anim[k].quad = {}
        for i=1,w/h do
            anim[k].quad[i] = love.graphics.newQuad((i-1)*h, 0, h, h, w, h)
        end
    end

    --particle systems

    --to the left of player
    psSplashL = love.graphics.newParticleSystem(tex.splash,256)
    psSplashL:setEmissionRate(0)
    psSplashL:setParticleLife(.5,.2)
    psSplashL:setSizeVariation(1)
    psSplashL:setSpin(5)
    psSplashL:setSpread(.2)
    psSplashL:setSpeed(150,150)
    psSplashL:setDirection(-math.pi + .5)
    psSplashL:start()
    psSplashL:setGravity(50,250)

    --to the right of player
    psSplashR = love.graphics.newParticleSystem(tex.splash,256)
    psSplashR:setEmissionRate(0)
    psSplashR:setParticleLife(.5,.2)
    psSplashR:setSizeVariation(1)
    psSplashR:setSpin(5)
    psSplashR:setSpread(.2)
    psSplashR:setSpeed(150,150)
    psSplashR:setDirection(-.5)
    psSplashR:start()
    psSplashR:setGravity(50,250)

    --hook splash
    psSplashHook = love.graphics.newParticleSystem(tex.splash,256)
    psSplashHook:setEmissionRate(0)
    psSplashHook:setParticleLife(.5,.2)
    psSplashHook:setSizeVariation(1)
    psSplashHook:setSpin(5)
    psSplashHook:setSpread(1)
    psSplashHook:setSpeed(150,150)
    psSplashHook:setDirection(-math.pi/2)
    psSplashHook:start()
    psSplashHook:setGravity(50,250)

    --hook drag
    psDragHook = love.graphics.newParticleSystem(tex.drag,256)
    psDragHook:setEmissionRate(0)
    psDragHook:setParticleLife(.5,.2)
    psDragHook:setSizeVariation(1)
    psDragHook:setSpin(5)
    psDragHook:setSpread(1)
    psDragHook:setSpeed(150,150)
    psDragHook:setDirection(-math.pi)
    psDragHook:start()
    psDragHook:setGravity(50,250)


    --idling boat
    boat = {
        x = 0,
    }


    --player
    tero = {
        x = screen.w/2,
        vx = 0,
    }

    hook = {
        x = screen.w/2,
        y = screen.h-240,
        vy = 0,
        --lastOnTop is used to tell when the hook hit the water, so it can splash
        lastOnTop = true,
        --fish ON the hook
        fish = {},
    }


    --sets up array for fish, 8 rows with up to 8 fish each
    fish = {}
    for i = 1,8 do
        fish[i] = {}
        for j = 1,8 do
            fish[i][j] = {
                type = 0,
                x = 0,
                right = false,
                --Random animation frame offset
                offset = 0,
            }
        end
    end
end










function love.update(dt2)
    --exit
    if love.keyboard.isDown('escape') then love.event.quit() end

    --make dt global
    dt = dt2

    --update game time thats passed total
    game.timer = game.timer + dt

    --update shaders
    shader.update.ripples()
    shader.update.sky()
    shader.update.sand()
    shader.update.lightrays()
    shader.update.reflection()
    shader.update.rain()


    --update particle systems
    psSplashL:update(dt)
    psSplashR:update(dt)
    psSplashHook:update(dt)
    psDragHook:update(dt)


    --updates when fish can spawn
    --more fish spawn the larger your screen
    --makes things more fair
    game.fishtimer = game.fishtimer + dt
    local tmp = 512/screen.w
    if game.fishtimer >= tmp then
        game.fishtimer = math.mod(game.fishtimer, tmp)
        spawnFish()
    end

    --cut down on time you have left
    game.score = math.clamp(game.score - dt*2, 0, 255)

    --solar powered flashlight
    if game.flashlight then
        game.flashlighttimer = math.clamp(game.flashlighttimer - dt*2, 0, 255)
    elseif math.mod(game.timer/32,32) <=8 then
        game.flashlighttimer = math.clamp(game.flashlighttimer + dt*2, 0, 255)
    end

    --moves idling boat
    boat.x = math.sin(math.mod(((512/screen.w)*game.timer)/8, math.pi*16)) * (screen.w - 128) / 2 + (screen.w - 128)/2

    --moves character
    if isDown('right') and tero.x <= screen.w - 128 then
        tero.vx = tero.vx + 1024*dt
    elseif isDown('left') and tero.x >= 0 then
        tero.vx = tero.vx - 1024*dt
    else
        tero.vx = tero.vx - (tero.vx * dt * 2)
        if math.abs(tero.vx) < 4 then tero.vx = 0 end
    end
    tero.vx = math.clamp(tero.vx, -256, 256)
    tero.x = tero.x + dt * tero.vx
    tero.x = math.clamp(tero.x, -128, screen.w)


    --moves hook
    --if the hook's underwater...
    if hook.y >= screen.h-240 then
        --"and hook.vy >=4" makes sure you can't stop a pull short and cheat
        if isDown('up') and hook.vy >= -4 then hook.vy = -4 end
        if isDown('pull') then hook.vy = -12 end
        hook.vy = math.clamp(hook.vy, -12, 2)
        if isDown('down') and hook.vy >= -4 then hook.vy = 4 end
        if not hook.lastOnTop then
            hook.lastOnTop = not hook.lastOnTop
            psSplashHook:setEmissionRate(50)
            if sfx.playing.splash == 0 then
                sfx.playing.splash = 1
            end
        else
            psSplashHook:setEmissionRate(0)
        end
    else
        if hook.lastOnTop then
            hook.lastOnTop = not hook.lastOnTop
            psSplashHook:setEmissionRate(50)
            if sfx.playing.up == 0 then
                sfx.playing.up = 1
            end
        else
            psSplashHook:setEmissionRate(0)
        end
    end


    --tries to put the hook at tero.x (the rear of the boat) instead of the middle to simulate drag
    hook.x = hook.x + (tero.x - hook.x) * dt * 2
    hook.vy = hook.vy + dt * 8
    hook.y = math.clamp(hook.y + hook.vy * dt * 32, screen.h - 544, screen.h - 64)


    --adds dragging particle effect
    if hook.y == screen.h - 64 then
        psDragHook:setEmissionRate(math.abs(math.ceil(tero.x-hook.x)+1))
    else
        psDragHook:setEmissionRate(0)
    end


    --attatches fish to the line
    local tmp = math.ceil((hook.y - (screen.h-240) + 23)/24)
    if hook.vy < -4 and tmp >= 1 and tmp <= 8 then
        for i = 1,8 do
            if fish[i][tmp].x >= hook.x - 24 - (tero.x-hook.x + 24)/4 and fish[i][tmp].x <= hook.x + 24 - (tero.x-hook.x + 24)/4 and fish[i][tmp].type ~= 0 then
                hook.fish[table.getn(hook.fish) + 1] = {
                    type = fish[i][tmp].type,
                    --the fishs' animation offset also determines its' random size
                    offset = fish[i][tmp].offset,
                    right = fish[i][tmp].right,
                }
                fish[i][tmp].type = 0
                if sfx.playing.cut == 0 then
                    sfx.playing.cut = 1
                end
            end
        end
    end


    --puts fish in the boat
    if hook.vy > 0 and hook.y >= screen.h-240 - 32 then
        if hook.x >= boat.x and hook.x <= boat.x + 128 then
            for i = 1, table.getn(hook.fish) do
                game.score = math.clamp(game.score + hook.fish[i].type * 16, 0, 255)
                game.totalscore = game.totalscore + hook.fish[i].type * 16
                if sfx.playing.slap == 0 then
                    sfx.playing.slap = 1
                end
            end
        end
        hook.fish = {}
    end


    --moves fish
    for i = 1,8 do
        for j = 1,8 do
            if fish[i][j].type ~= 0 then
                if fish[i][j].right then
                    fish[i][j].x = fish[i][j].x + (dt*64) * (1 + fish[i][j].type/4)
                else
                    fish[i][j].x = fish[i][j].x - (dt*64) * (1 + fish[i][j].type/4)
                end
            end
        end
    end


    --despawn fish
    --because fish start out at 0 or 1, and they move BEFORE
    --this code, they can safely be despawned
    for i = 1,8 do
        for j = 1,8 do
            if fish[i][j].x <= 0 or fish[i][j].x >= screen.w + 64 then
                fish[i][j].type = 0
            end
        end
    end




    --audio

    if settings.sfx then

        --flashlight click
        if sfx.playing.switch == 1 then
            love.audio.play(sfx.switch)
            sfx.playing.switch = sfx.playing.switch - dt
        elseif sfx.playing.switch > 0 then
            sfx.playing.switch = sfx.playing.switch - dt
        elseif sfx.playing.switch < 0 then
            love.audio.stop(sfx.switch)
            sfx.playing.switch = 0
        end

        --rain
        if game.rain > 0 then
            sfx.rain:setVolume(game.rain)
            sfx.boatrain:setVolume(game.rain)
            --boatrain and rain are tied and thus dont need their own sfx.playing variable
            if not sfx.playing.rain then
                sfx.playing.rain = true
                love.audio.play(sfx.rain)
                sfx.playing.boatrain = true
                love.audio.play(sfx.boatrain)
            end
            sfx.boatrain:setPosition(tero.x/screen.w - .5, 0, 0)
        elseif sfx.playing.rain then
            sfx.playing.rain = false
            if sfx.playing.rain then
                love.audio.stop(sfx.rain)
                love.audio.stop(sfx.boatrain)
                sfx.playing.rain = false
            end
        end

        --reeling
        if hook.vy < 0 then
            sfx.reel:setVolume(-hook.vy/16)
            sfx.reel:setPitch(-hook.vy/10)
            if not sfx.playing.reel then
                sfx.playing.reel = true
                love.audio.play(sfx.reel)
            end
            sfx.reel:setPosition(hook.x/screen.w - .5, 0, 0)
        elseif sfx.playing.reel then
            sfx.playing.reel = false
            love.audio.stop(sfx.reel)
        end

        --moving your boat
        if math.abs(tero.vx) > 0 then
            sfx.surf:setVolume(math.abs(tero.vx/256))
            if not sfx.playing.surf then
                sfx.playing.surf = true
                love.audio.play(sfx.surf)
            end
            sfx.surf:setPosition(tero.x/screen.w - .5, 0, 0)
        elseif sfx.playing.surf then
            sfx.playing.surf = false
            love.audio.stop(sfx.surf)
        end

        --wind
        if hook.y < screen.h-240 then
            sfx.wind:setVolume(math.abs(hook.vy)/64 + math.abs(hook.x-tero.x)/256)
            sfx.wind:setPitch(.5 + math.abs(hook.vy)/10 + math.abs(hook.x-tero.x)/128)
            if not sfx.playing.wind then
                sfx.playing.wind = true
                love.audio.play(sfx.wind)
            end
            sfx.wind:setPosition(hook.x/screen.w - .5, 0, 0)
        elseif sfx.playing.wind then
            sfx.playing.wind = false
            love.audio.stop(sfx.wind)
        end

        --splash down
        if sfx.playing.splash == 1 then
            sfx.splash:setPosition(hook.x/screen.w - .5, 0, 0)
            love.audio.rewind(sfx.splash)
            love.audio.play(sfx.splash)
            sfx.playing.splash = sfx.playing.splash - dt
        elseif sfx.playing.splash > 0 then
            sfx.playing.splash = sfx.playing.splash - dt
        else
            sfx.playing.splash = 0
        end

        --splash up
        if sfx.playing.up == 1 then
            sfx.up:setPosition(hook.x/screen.w - .5, 0, 0)
            love.audio.rewind(sfx.up)
            love.audio.play(sfx.up)
            sfx.playing.up = sfx.playing.up - dt
        elseif sfx.playing.up > 0 then
            sfx.playing.up = sfx.playing.up - dt
        else
            sfx.playing.up = 0
        end

        --fish snag
        if sfx.playing.cut == 1 then
            sfx.cut:setPosition(hook.x/screen.w - .5, 0, 0)
            love.audio.rewind(sfx.cut)
            love.audio.play(sfx.cut)
            sfx.playing.cut = sfx.playing.cut - dt
        elseif sfx.playing.cut > 0 then
            sfx.playing.cut = sfx.playing.cut - dt
        else
            sfx.playing.cut = 0
        end

        --fish hitting boat
        if sfx.playing.slap == 1 then
            sfx.slap:setPosition(hook.x/screen.w - .5, 0, 0)
            love.audio.rewind(sfx.slap)
            love.audio.play(sfx.slap)
            sfx.playing.slap = sfx.playing.slap - dt
        elseif sfx.playing.slap > 0 then
            sfx.playing.slap = sfx.playing.slap - dt
        else
            sfx.playing.slap = 0
        end


    end



    --rain, ~4 minute cycle
    local tmp = math.mod(game.timer/128,8)
    if math.ceil(tmp) == 2 then
        game.rain = math.clamp(math.mod(tmp, 1), 0, 1)
    elseif math.ceil(tmp) == 3 then
        game.rain = 1
    elseif math.ceil(tmp) == 4 then
        game.rain = 1 - math.clamp(math.mod(tmp, 1), 0, 1)
    elseif math.ceil(tmp) == 5 then
        game.rain = 0
    end


end










function love.draw()
    love.graphics.setCanvas(shader.canvas[1])
    love.graphics.setBlendMode('premultiplied')

    --background black
    love.graphics.setColor(0,0,0)
    love.graphics.rectangle('fill', 0, 0, screen.w, screen.h)
    love.graphics.setColor(255,255,255)

    --mode 7 sky
    shader.draw.sky()

    --island
    love.graphics.draw(island[1], 640 - math.mod(game.timer/64, 3)*640, screen.h-480)

    --mode 7 sand
    shader.draw.sand()

    --water fake fog
    love.graphics.draw(tex.shadow, 0, screen.h-480, 0, screen.w, 1)
    love.graphics.draw(tex.shadow, 0, screen.h-480, 0, screen.w, 1)
    love.graphics.draw(tex.shadow, 0, screen.h-480, 0, screen.w, 1)
    love.graphics.draw(tex.shadow, 0, screen.h-480, 0, screen.w, 1)
    love.graphics.draw(tex.shadow, 0, screen.h-480, 0, screen.w, 1)
    love.graphics.draw(tex.shadow, 0, screen.h-480, 0, screen.w, 1)
    love.graphics.draw(tex.shadow, 0, screen.h-480, 0, screen.w, 1)
    --yeah ive been playing with the strength for testing purposes

    --particle systems for the player
    love.graphics.setColor(255, 255, 255, 255)
    if tero.vx > 0 then
        psSplashL:setEmissionRate(tero.vx)
        psSplashL:setSpeed(tero.vx/4)
    else
        psSplashL:setEmissionRate(0)
    end
    if tero.vx < 0 then
        psSplashR:setEmissionRate(math.abs(tero.vx))
        psSplashR:setSpeed(math.abs(tero.vx)/4)
    else
        psSplashR:setEmissionRate(0)
    end
    love.graphics.draw(psSplashL, tero.x + 24, screen.h-240)
    love.graphics.draw(psSplashR, tero.x + 104, screen.h-240)
    love.graphics.draw(psSplashHook, hook.x, screen.h-240)
    love.graphics.draw(psDragHook, hook.x, screen.h-8)
    love.graphics.draw(tex.boat, tero.x, screen.h-272)

    --rock
    love.graphics.draw(tex.rock, screen.w - math.mod(game.timer/32, 2)*1024, screen.h-172)

    --fishing line
    love.graphics.setColor(128,128,128, 255)
    local x1 = tero.x + 64
    local y1 = screen.h-272
    local x2 = hook.x - 4
    local y2 = hook.y + 12
    local tablelol = {}
    tablelol[1] = x1
    tablelol[2] = y1
    tablelol[15] = x2
    tablelol[16] = y2
    for i = 3,14,2 do
        tablelol[i]   = x1 - (x1-x2) * i/16
        tablelol[i+1] = y1*(1-i/16)+y2*(i/16)^.5
    end
    love.graphics.line(tablelol)
    love.graphics.setColor(255,255,255)

    --hook
    love.graphics.draw(tex.hook, hook.x, hook.y + 16, (tero.x-hook.x + 24)/256, -math.sin(tero.vx/256) + .1, 1, 24, 8)

    --fish
    for i = 1,8 do
        for j = 1,8 do
            if fish[i][j].type ~= 0 then
                if fish[i][j].right then
                    love.graphics.drawq(anim.fish.tex, anim.fish.quad[math.clamp(math.ceil(math.mod(game.timer*8 + fish[i][j].offset, 8)), 1, 8)], fish[i][j].x, j*24+screen.h-240, 0,  1 - fish[i][j].offset/16, 1 - fish[i][j].offset/16, 32, 16)
                else
                    love.graphics.drawq(anim.fish.tex, anim.fish.quad[math.clamp(math.ceil(math.mod(game.timer*8 + fish[i][j].offset, 8)), 1, 8)], fish[i][j].x, j*24+screen.h-240, 0, -1 + fish[i][j].offset/16, 1 - fish[i][j].offset/16, 32, 16)
                end
            end
        end
    end

    --fish on the hook
    for i = 1, table.getn(hook.fish) do
        if hook.fish[i].right then
            love.graphics.drawq(anim.fish.tex, anim.fish.quad[1], hook.x + 32 - (tero.x-hook.x + 24)/4, hook.y + 32, 0, 1 - hook.fish[i].offset/16, 1 - hook.fish[i].offset/16, 64, 0)
        else
            love.graphics.drawq(anim.fish.tex, anim.fish.quad[1], hook.x + 32 - (tero.x-hook.x + 24)/4, hook.y + 32, 0, -1 + hook.fish[i].offset/16, 1 - hook.fish[i].offset/16, 0, 0)
        end
    end

    --other boat
    love.graphics.setColor(128, 64, 64, 255)
    love.graphics.draw(tex.boat, boat.x, screen.h-272)
    love.graphics.setColor(255, 255, 255, 255)

    --floater... things in the water
    for i = 1, math.ceil(screen.w/256) + 1 do
        for j = 1, math.ceil((screen.h-240)/256) do
            love.graphics.setColor(multiplyChannels(255,255,255,64))
            love.graphics.draw(tex.float, (i-1)*256 - math.mod(game.timer, 4) * 64, screen.h-240)
            love.graphics.setColor(multiplyChannels(255,255,255,128))
            love.graphics.draw(tex.float, (i-1)*256 - math.mod(game.timer, 8) * 32, screen.h-240 + 32)
        end
    end
    love.graphics.setColor(255,255,255,255)

    --this actually does ripples and the reflection
    --i will later change it to 'reflect' that
    shader.draw.ripples()
    shader.draw.rain()
    shader.draw.day()
    shader.draw.hud()

    love.graphics.setCanvas()
    love.graphics.draw(shader.canvas[1], 0, 0)

    love.graphics.setColor(192, 164, 164)
    love.graphics.printf(love.timer.getFPS(), 44, 0, 4, 'center')
    love.graphics.printf(game.totalscore, screen.w-44, 0, 4, 'center')
    love.graphics.setColor(255, 255, 255)
end










function spawnFish()
    for i = 1,8 do
        ran1 = math.random(1,8)
        ran2 = math.random(1,8)
        if fish[ran1][ran2].type == 0 then
            if math.random(1, 2) == 1 then
                fish[ran1][ran2].type = math.random(1,4)
                fish[ran1][ran2].right = true
                fish[ran1][ran2].x = 0
                fish[ran1][ran2].offset = math.random(1,7)
            else
                fish[ran1][ran2].type = math.random(1,4)
                fish[ran1][ran2].right = false
                fish[ran1][ran2].x = screen.w + 64
                fish[ran1][ran2].offset = math.random(1,7)
            end
            return
        end
    end
end


function isDown(key)
    if love.keyboard.isDown(unpack(settings.button[key])) then
        return true
    end
    return false
end


function love.keypressed(key)
    if key == 'q' then debugging = not debugging end
    if isDown('flashlight') then
        game.flashlight = not game.flashlight
        if sfx.playing.switch ~= 0 then
            love.audio.stop(sfx.switch)
        end
        sfx.playing.switch = 1
    end
    if key == 'f10' then fullscreen() end
end


function fullscreen()
    if settings.fullscreen then
        settings.fullscreen = false
        love.graphics.setMode(640, 480, false, false)
        screen = {
            w = 640,
            h = 480,
        }
    else
        settings.fullscreen = true
        love.graphics.setMode(settings.modes[1].width,settings.modes[1].height, true, false)
        screen = {
            w = settings.modes[1].width,
            h = settings.modes[1].height,
        }
    end
end